@@ -63,10 +63,33 @@ module Y2Storage
     storage_forward :key_file=
 
     # @!attribute cipher
-    #   @return [String] the encryption cipher
+    #   The encryption cipher
+    #
+    #   Currently only supported for LUKS
+    #
+    #   @return [String] if empty, the default of cryptsetup will be used during creation
     storage_forward :cipher
     storage_forward :cipher=
 
+    # @!attribute key_size
+    #   The key size in bytes
+    #
+    #   Currently only supported for LUKS
+    #
+    #   Note the value is expressed in bytes. That's dictated by libstorage-ng, even when cryptsetup
+    #   and all the LUKS-related documentation use bits for expressing the key size.
+    #
+    #   @return [Integer] if zero, the default of cryptsetup will be used during creation
+    storage_forward :key_size
+    storage_forward :key_size=
+
+    # @!attribute pbkdf_value
+    #   String representation of {#pbkdf}, an empty string is equivalent to a nil value on {#pbkdf}
+    #
+    #   @return [String]
+    storage_forward :pbkdf_value, to: :pbkdf
+    storage_forward :pbkdf_value=, to: :pbkdf=
+
     # @!attribute crypt_options
     #   Options in the fourth field of /etc/crypttab
     #
@@ -98,6 +121,10 @@ module Y2Storage
     storage_forward :storage_in_etc_crypttab=, to: :in_etc_crypttab=
     private :storage_in_etc_crypttab=
 
+    # @!method set_default_mount_by
+    #   Set the mount-by method to the global default, see Storage::get_default_mount_by()
+    storage_forward :set_default_mount_by, to: :default_mount_by=
+
     # @see BlkDevice#plain_device
     def plain_device
       blk_device
@@ -365,11 +392,80 @@ module Y2Storage
       end
     end
 
+    # Sets {#mount_by} to a value that makes sense
+    #
+    # Generally, that means simply ensuring a suitable mount_by value. But it also may imply
+    # correcting the surprising value set by libstorage-ng when probing an already existing
+    # encryption device.
+    #
+    # During probing, libstorage-ng sets Encryption#mount_by for all the found encryption devices.
+    # If the device is listed in /etc/crypttab, libstorage-ng sets the mount_by value based on the
+    # value on that file. If that's not the case, libstorage-ng sets Encryption#mount_by to a
+    # hardcoded value of DEVICE, completely ignoring the default mount_by value that is configured
+    # for the system. That leads to problems like the one described in bsc#1165702.
+    # See https://github.com/yast/yast-storage-ng/pull/1095 for more details.
+    #
+    # For devices affected by that problem, this method first tries to re-initialize mount_by to
+    # a value aligned with the storage-ng configuration and, thus, with user expectations.
+    def adjust_mount_by
+      # It may be more correct to do this only once (since it's fixing a wrong initialization).
+      # But since there is no way in the UI or AutoYaST to explicitly modify (even to display) the
+      # value of Encryption#mount_by, it's safe (even maybe expected) to re-evaluate it.
+      set_default_mount_by if probed_without_crypttab?
+
+      ensure_suitable_mount_by
+    end
+
     # @see Device#update_etc_attributes
     def assign_etc_attribute(value)
       self.storage_in_etc_crypttab = value
     end
 
+    # PBKDF (Password-Based Key Derivation Function), currently only supported for LUKS2 where
+    # this attribute corresponds to the PBKDF of the first used keyslot.
+    #
+    # If is set to nil, during the commit phase the default of cryptsetup will be used.
+    #
+    # @return [PbkdFunction, nil]
+    def pbkdf
+      PbkdFunction.find(pbkdf_value)
+    end
+
+    # @see #pbkdf
+    #
+    # @param function [PbkdFunction, nil]
+    def pbkdf=(function)
+      self.pbkdf_value = function&.value || ""
+    end
+
+    # Whether the attribute #pbkdf makes sense for this object
+    #
+    # @return [Boolean]
+    def supports_pbkdf?
+      type.is?(:luks2)
+    end
+
+    # Whether the attribute #label makes sense for this object
+    #
+    # @return [Boolean]
+    def supports_label?
+      type.is?(:luks2)
+    end
+
+    # Whether the attribute #cipher makes sense for this object
+    #
+    # @return [Boolean]
+    def supports_cipher?
+      type.is?(:luks1, :luks2)
+    end
+
+    # Whether the attribute #key_size makes sense for this object
+    #
+    # @return [Boolean]
+    def supports_key_size?
+      type.is?(:luks1, :luks2)
+    end
+
     protected
 
     # @see Device#is?
@@ -440,6 +536,19 @@ module Y2Storage
       mount_point.path.gsub(/^\/|\/$/, "").gsub("/", "_")
     end
 
+    # Checks whether the initial value of mount_by was forced to DEVICE by libstorage-ng due to lack
+    # of information or whether is the result of a proper initialization during normal operation
+    #
+    # @see #adjust_mount_by
+    #
+    # @return [Boolean]
+    def probed_without_crypttab?
+      return false unless exists_in_probed?
+      return !in_etc_crypttab? if in_etc_initial.nil?
+
+      !in_etc_initial
+    end
+
     class << self
       # Updates the DeviceMapper name for all encryption devices in the device
       # that have a name automatically set by YaST.
@@ -464,7 +573,7 @@ module Y2Storage
         end
 
         # ...reassign them according to the current names of the block devices
-        encryptions.each do |enc|
+        encryptions.each do |enc| # rubocop:disable Style/CombinableLoops Cannot be combined
           enc.assign_dm_table_name(enc.auto_dm_table_name)
         end
       end
